/*
 *  This file is part of uIRC. (https://git.redxen.eu/caskd/uIRC)
 *  Copyright (c) 2019, 2020 Alex-David Denes
 *
 *  uIRC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  uIRC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uIRC.  If not, see <https://www.gnu.org/licenses/>.
 */

/*! \file */

#include "types.h"

#include <sys/types.h>

#ifndef UIRC_GUARD_ASSEMBLERS
#define UIRC_GUARD_ASSEMBLERS

signed long Assm_user(char* buf, IRC_User* in, size_t len, bool useorig);

#ifdef UIRC_IRCV3
signed long Assm_tags(char* buf, IRC_Tags* in, size_t len);
#endif /* UIRC_IRCV3*/

/*!
 * \brief IRC_Message to string converter
 *
 * This assembles a IRC_Message struct to a string representation of a most size len
 * The return value is either the lenght of the string or any of the uirc_errors errors
 * \param[in] in IRC_Message struct that contains at least a command
 * \param[in] len Maximum lenght of string
 * \param[out] buf String representation of message (if successful)
 * \warning in and buf SHOULD NOT be NULL, this will trigger a ASSERT so it's required to check it where needed
 */
signed long Assm_mesg(char* buf, IRC_Message* in, size_t len);

#ifdef UIRC_HELPERS
IRC_Message* Assm_AUTO(IRC_Message* imassm_mesg, IRC_Command cmd, bool trailing, char** args, int req);
IRC_Message* Assm_cmd_USER(IRC_Message* imassm_mesg, char* user, char* realname, int modes);
IRC_Message* Assm_cmd_LINKS(IRC_Message* imassm_mesg, char* remoteserv, char* servmask);
IRC_Message* Assm_cmd_WHO(IRC_Message* imassm_mesg, char* mask, bool oper);
IRC_Message* Assm_cmd_WHOIS(IRC_Message* imassm_mesg, char* target, char* mask);
IRC_Message* Assm_cmd_WHOWAS(IRC_Message* imassm_mesg, char* nick, char* count, char* target);
IRC_Message* Assm_cmd_PING(IRC_Message* imassm_mesg, char* source, char* target);
IRC_Message* Assm_cmd_SUMMON(IRC_Message* imassm_mesg, char* user, char* target, char* channel);
IRC_Message* Assm_cmd_USERHOST(IRC_Message* imassm_mesg, char* users[]);
IRC_Message* Assm_cmd_ISON(IRC_Message* imassm_mesg, char* users[]);

#define Assm_cmd_REHASH(mesg)  Assm_AUTO(mesg, REHASH, false, (char*[]) { NULL }, 0)
#define Assm_cmd_DIE(mesg)     Assm_AUTO(mesg, DIE, false, (char*[]) { NULL }, 0)
#define Assm_cmd_RESTART(mesg) Assm_AUTO(mesg, RESTART, false, (char*[]) { NULL }, 0)

#define Assm_cmd_QUIT(mesg, message)   Assm_AUTO(mesg, QUIT, true, (char*[]) { message, NULL }, 0)
#define Assm_cmd_MOTD(mesg, target)    Assm_AUTO(mesg, MOTD, false, (char*[]) { target, NULL }, 0)
#define Assm_cmd_VERSION(mesg, target) Assm_AUTO(mesg, VERSION, false, (char*[]) { target, NULL }, 0)
#define Assm_cmd_TIME(mesg, target)    Assm_AUTO(mesg, TIME, false, (char*[]) { target, NULL }, 0)
#define Assm_cmd_TRACE(mesg, target)   Assm_AUTO(mesg, TRACE, false, (char*[]) { target, NULL }, 0)
#define Assm_cmd_ADMIN(mesg, target)   Assm_AUTO(mesg, ADMIN, false, (char*[]) { target, NULL }, 0)
#define Assm_cmd_INFO(mesg, target)    Assm_AUTO(mesg, INFO, false, (char*[]) { target, NULL }, 0)
#define Assm_cmd_AWAY(mesg, message)   Assm_AUTO(mesg, AWAY, false, (char*[]) { message, NULL }, 0)

#define Assm_cmd_PASS(mesg, password) Assm_AUTO(mesg, PASS, true, (char*[]) { password, NULL }, 1)
#define Assm_cmd_ERROR(mesg, message) Assm_AUTO(mesg, ERROR, true, (char*[]) { message, NULL }, 1)
#define Assm_cmd_WALLOPS(mesg, text)  Assm_AUTO(mesg, WALLOPS, true, (char*[]) { text, NULL }, 1)
#define Assm_cmd_NICK(mesg, nickname) Assm_AUTO(mesg, NICK, false, (char*[]) { nickname, NULL }, 1)
#define Assm_cmd_USERS(mesg, target)  Assm_AUTO(mesg, USERS, false, (char*[]) { target, NULL }, 1)

#define Assm_cmd_NAMES(mesg, channels, target) Assm_AUTO(mesg, NAMES, false, (char*[]) { channels, target, NULL }, 0)
#define Assm_cmd_LIST(mesg, channels, target)  Assm_AUTO(mesg, LIST, false, (char*[]) { channels, target, NULL }, 0)
#define Assm_cmd_LUSERS(mesg, mask, target)    Assm_AUTO(mesg, LUSERS, false, (char*[]) { mask, target, NULL }, 0)
#define Assm_cmd_STATS(mesg, query, target)    Assm_AUTO(mesg, STATS, false, (char*[]) { query, target, NULL }, 0)
#define Assm_cmd_SERVLIST(mesg, mask, type)    Assm_AUTO(mesg, SERVLIST, false, (char*[]) { mask, type, NULL }, 0)

#define Assm_cmd_JOIN(mesg, channels, keys)   Assm_AUTO(mesg, JOIN, false, (char*[]) { channels, keys, NULL }, 1)
#define Assm_cmd_PART(mesg, channel, message) Assm_AUTO(mesg, PART, false, (char*[]) { channel, message, NULL }, 1)
/* NOTE: Use a non-NULL address (mesg, pointing at a "\0") as the topic to clear it and use a NULL address to check it
 *       Blame the protocol, not this >:C */
#define Assm_cmd_TOPIC(mesg, channel, topic) Assm_AUTO(mesg, TOPIC, true, (char*[]) { channel, topic, NULL }, 1)
#define Assm_cmd_PONG(mesg, source, target)  Assm_AUTO(mesg, PONG, true, (char*[]) { source, target, NULL }, 1)

#define Assm_cmd_OPER(mesg, name, password)      Assm_AUTO(mesg, OPER, true, (char*[]) { name, password, NULL }, 2)
#define Assm_cmd_SQUIT(mesg, server, comment)    Assm_AUTO(mesg, SQUIT, true, (char*[]) { server, comment, NULL }, 2)
#define Assm_cmd_PRIVMSG(mesg, target, message)  Assm_AUTO(mesg, PRIVMSG, true, (char*[]) { target, message, NULL }, 2)
#define Assm_cmd_NOTICE(mesg, target, text)      Assm_AUTO(mesg, NOTICE, true, (char*[]) { target, text, NULL }, 2)
#define Assm_cmd_SQUERY(mesg, servicename, text) Assm_AUTO(mesg, SQUERY, true, (char*[]) { servicename, text, NULL }, 2)
#define Assm_cmd_KILL(mesg, nick, comment)       Assm_AUTO(mesg, KILL, true, (char*[]) { nick, comment, NULL }, 2)
#define Assm_cmd_INVITE(mesg, nick, channel)     Assm_AUTO(mesg, INVITE, false, (char*[]) { nick, channel, NULL }, 2)

#define Assm_cmd_MODE(mesg, nickname, modes, modeparams) Assm_AUTO(mesg, MODE, false, (char*[]) { nickname, modes, modeparams, NULL }, 1)

#define Assm_cmd_KICK(mesg, channels, users, comment) Assm_AUTO(mesg, KICK, true, (char*[]) { channels, users, comment, NULL }, 2)
#define Assm_cmd_CONNECT(mesg, target, port, remote)  Assm_AUTO(mesg, CONNECT, false, (char*[]) { target, port, remote, NULL }, 2)

#define Assm_cmd_SERVICE(mesg, nickname, distribution, type, info) \
	Assm_AUTO(mesg, SERVICE, true, (char*[]) { nickname, RESERVED, distribution, "0", RESERVED, info, NULL }, 6)

#ifdef UIRC_IRCV3
#define Assm_cmd_CAP_END(mesg)  Assm_AUTO(mesg, CAP, false, (char*[]) { "END", NULL }, 0)
#define Assm_cmd_CAP_LIST(mesg) Assm_AUTO(mesg, CAP, false, (char*[]) { "LIST", NULL }, 0)

#define Assm_cmd_CAP_LS(mesg, version) Assm_AUTO(mesg, CAP, false, (char*[]) { "LS", version, NULL }, 0)
#define Assm_cmd_CAP_REQ(mesg, caps)   Assm_AUTO(mesg, CAP, true, (char*[]) { "REQ", caps, NULL }, 1)

#define Assm_cmd_CAP_NEW(mesg, nick, caps) Assm_AUTO(mesg, CAP, true, (char*[]) { "NEW", nick, caps, NULL }, 2)
#define Assm_cmd_CAP_DEL(mesg, nick, caps) Assm_AUTO(mesg, CAP, true, (char*[]) { "DEL", nick, caps, NULL }, 2)
#endif /* UIRC_IRCV3 */
#endif /* UIRC_HELPERS */

#endif /* UIRC_GUARD_ASSEMBLERS */

